<!-- The Begin and End comments throughout this document are used in order to pull specific sections of the readme into the main GUI window at runtime. -->

# Randovania

Welcome to Randovania, a randomizer platform for a multitude of games.

New here or looking to install? Check [our website](https://randovania.github.io/).

<!-- Begin SUPPORTED -->

### Supported Games
 - Another Metroid 2 Remake
 - Cave Story
 - Metroid Dread
 - Metroid Prime
 - Metroid Prime 2: Echoes

<!-- End SUPPORTED -->

<!-- Begin EXPERIMENTAL -->

### Experimental Games
 - Metroid Prime 3: Corruption
 - Metroid: Samus Returns
 - Super Metroid

<!-- End EXPERIMENTAL -->

<!-- Begin WELCOME -->

Here you will be able to randomize many aspects of either game, while still being ensured it's possible to 
finish without any trick or glitch! What can be randomized?

* Randomize what can be found in each pickup location, including major upgrades, expansions, keys and artifacts.

* Play with multiple people, via multiworld sessions. Your pickups will be shuffled among the games of
everyone involved, no matter what game they're playing!

* Randomize where each teleporter goes, or what you need to unlock a translator gate. In either case, 
there's advanced options for how they're shuffled.

* The location you start the game in as well as the items you start with. If you're feeling brave, 
you can even shuffle items you normally start with, like the Power Beam and Scan Visor.

So have fun and start randomizing.

<!-- End WELCOME -->

# Installation

## Windows

In the [releases page](https://github.com/randovania/randovania/releases), we have zip files
with everything ready to use. Just extract and run!

## Linux

### Running (in-source)

1. Clone this repository (downloading the source zip is not supported and will not work)
2. Open a terminal in the repository root
3. Fetch the tags from the repo:

   `$ git fetch --tags`
4. Checkout the latest tag (replace v4.0.1 with the latest release version):

   `$ git checkout v4.0.1`
5. Prepare the virtual environment and install dependencies:
   
   `$ tools/prepare_virtual_env.sh`
6. Run the program from source:

   `$ tools/start_client.sh`

### Running Natively and Multiworld

1. Activate the virtual environment and generate the necessary configuration.json, as well as build a native application:

   ```console
   $ . venv/bin/activate
   (venv) $ export PRODUCTION=true
   (venv) $ python tools/create_release.py
   ```
2. Navigate into the build/randovania directory
3. Install the generated randovania.pkg or run the generated executable

<!-- Begin COMMUNITY -->

# Community

Join the Randovania Discord: <https://discord.gg/M23gCxj6fw>

Invite links for specific games' servers can be found in the `#game-communities` channel in our server.

<!-- End COMMUNITY -->

<!-- Begin CREDITS -->

# Credits

GUI and logic written by [Henrique Gemignani](https://github.com/henriquegemignani/), with contributions 
by [SpaghettiToastBook](https://www.twitch.tv/spaghettitoastbook), [gollop](https://github.com/gollop) and [many others](https://github.com/randovania/randovania/graphs/contributors).

[BashPrime](https://www.twitch.tv/bashprime), [Pwootage](https://github.com/Pwootage), and [April Wade](https://github.com/aprilwade) made <https://randomizer.metroidprime.run/>, from which the GUI was based.

Website created by [Hugoshido](https://twitch.tv/hugoshido).

Installer is powered by [Advanced Installer](https://www.advancedinstaller.com/), which has graciously provided us with an open source license.

Linux Flatpak build contributed by [Ethan Lee](https://flibitijibibo.com/).

## Games

### Metroid Prime 1
* Game patching via [randomprime](https://github.com/randovania/randomprime). Originally authored by [April Wade](https://github.com/aprilwade), it is now maintained and developed by [toasterparty](https://github.com/toasterparty) with contributions from [others](https://github.com/randovania/randomprime/graphs/contributors)
* Room data collected by UltiNaruto, [EthanArmbrust](https://github.com/EthanArmbrust) and [SolventMercury](https://github.com/SolventMercury).
* Converting Metroid Prime 2 models by [Migs](https://www.twitch.tv/migslive).

### Metroid Prime 2: Echoes
* Game patching written by [Claris](https://www.twitch.tv/claris).
* Room data initially collected by Claris, revamped by [Dyceron](https://www.twitch.tv/dyceron).
* [Menu Mod](https://www.dropbox.com/s/yhqqafaxfo3l4vn/Echoes%20Menu.7z) created by Claris. For more information, see the
[Menu Mod README](https://www.dropbox.com/s/yhqqafaxfo3l4vn/Echoes%20Menu.7z?file_subpath=%2FEchoes+Menu%2Freadme.txt).
* Converting Metroid Prime models by [Migs](https://www.twitch.tv/migslive).

### Metroid Prime 3: Corruption
* Game patching written by [gollop](https://github.com/gollop).
* Room data collected by [Dyceron](https://www.twitch.tv/dyceron) and [KirbymastaH](https://www.twitch.tv/kirbymastah).

### Super Metroid
* Game Patching and Logic Database by [SolventMercury](https://github.com/SolventMercury).
* Custom Item PLMs patch by [Kazuto](https://github.com/Kazuto88).
* Skip Intro Saves patch by [PHOSPHOTiDYL](https://metroidconstruction.com/resource.php?id=265).
* Other individual patches by [Total](https://github.com/tewtal), Foosda, Leodox, and others.

### Cave Story
* Patcher and logic written by [duncathan_salt](https://twitter.com/duncathan_salt).
* Based on the [original randomizer](https://shru.itch.io/cave-story-randomizer) by shru.
* Features contributions from [many others](https://github.com/cave-story-randomizer/cave-story-randomizer/graphs/contributors).

### Metroid Dread
* Game Patching by:
  * [Henrique "Darkszero" Gemignani](https://github.com/henriquegemignani/)
  * [duncathan_salt](https://twitter.com/duncathan_salt)
  * [ScorelessPine](https://github.com/ScorelessPine)
  * [Arcanox](https://twitter.com/ArcanoxDragon)
  * [Migs](https://www.twitch.tv/migslive)
  * [hyperbola0](https://github.com/steven11sjf)
  * [Thanatos](https://github.com/ThanatosGit)

* Logic Database by:
  * [KirbymastaH](https://www.twitch.tv/kirbymastah)
  * [Dyceron](https://www.twitch.tv/dyceron)
  * [XenoWars](https://www.twitch.tv/xenowars1)
  * [Mayberry](https://github.com/Isachu)
  * [Hugoshido](https://twitch.tv/hugoshido)
  * [Tyranisaur](https://github.com/Tyranisaur)

* Assets by:
  * Morph Ball and Speed Booster pickup textures created by [BigSharkZ](https://www.youtube.com/BigSharkZ).
  * Spider Magnet pickup texture by duncathan_salt with help from BigSharkZ.
  * New map icons by [SkyTheLucario](https://github.com/TheSkyknight100).

### Another Metroid 2 Remake
* Game Patching by:
  * [Miepee](https://github.com/Miepee)
  * [JesRight](https://github.com/Jesright73)
  
* Logic Database by:
  * [Miepee](https://github.com/Miepee)
  * [DruidVorse](https://www.youtube.com/@DruidVorse)
  * [JeffGainsNGames](https://www.youtube.com/@jeffgainsngames)

* Assets by:
  * Morph Ball, and the Missile Launcher sprites were made by ShirtyScarab554 licensed under [CC BY-SA 4.0](https://creativecommons.org/licenses/by-sa/4.0/).
  * Power Grip and the Shiny Nothing Orb were made by ShirtyScarab554, used under [CC BY-SA 4.0](https://creativecommons.org/licenses/by-sa/4.0/), modified by [AbyssalCreature](https://github.com/AbyssalCreature) and licensed under [CC BY-SA 4.0](https://creativecommons.org/licenses/by-sa/4.0/).
  * New door sprites and other AM2R item sprites were made by [AbyssalCreature](https://github.com/AbyssalCreature) licensed under [CC BY-SA 4.0](https://creativecommons.org/licenses/by-sa/4.0/).
  
## Auto Tracker
* Game theme assets were provided by [MaskedTAS](https://twitter.com/MaskedTAS).
* Pixel theme assets were provided by [Uncle Reggie](https://www.twitch.tv/unclereggie).
* AM2R 1.5.5 item sprites were made by [Eskimode7](https://twitter.com/shmegleskimo) licensed under [CC BY-SA 4.0](https://creativecommons.org/licenses/by-sa/4.0/).
* The AM2R DNA sprite was made by [AbyssalCreature](https://github.com/AbyssalCreature) licensed under [CC BY-SA 4.0](https://creativecommons.org/licenses/by-sa/4.0/).
* The AM2R Morph Ball and Power Grip sprites were made by ShirtyScarab554 licensed under [CC BY-SA 4.0](https://creativecommons.org/licenses/by-sa/4.0/).

## Multiworld
Server and logic written by Henrique, including Dolphin and Nintendont integrations. These were based on [Dolphin Memory Engine](https://github.com/aldelaro5/Dolphin-memory-engine) and Pwootage's Nintendont fork, respectively. In-game message alert initially written by [encounter](https://github.com/encounter).

<!-- End CREDITS -->

# Developer Help

## Dependencies

* [Python 3.11 64-bit](https://www.python.org/ftp/python/3.11.0/python-3.11.0rc1-amd64.exe)
* [Git](https://git-scm.com/downloads)

## Setup

Getting started:
   1. Clone this repository (downloading the zip is *not* supported and will not work)
   2. Open a terminal in the repository root
   3. Run the following file:
      1. Windows: `tools/prepare_virtual_env.bat`
      2. Linux/macOS: `tools/prepare_virtual_env.sh`
   4. You should see "Setup finished successfully." visible when the command finishes.

In order to start Randovania, open:
   1. Windows: `tools/start_client.bat`
   2. Linux/macOS: `tools/start_client.sh` 

In order to update your repository:
   1. Update the git repository. (With `git pull` or anything else)
   2. Make sure that Randovania is closed.
   3. Re-run the steps from "Getting Started", starting at step 2.
      1. In case of unexpected errors, delete the `venv` in the root of the repository and start again.
   4. Open Randovania normally.

In order to run the tests:
   1. Run both "Getting started" and "Start Randovania" steps. 
   2. Activate the virtual env. Check start_client.bat/sh for details.
   3. Run `python -m pip install -r requirements.txt`.
   4. Run `python -m pytest test`.

In order to run the server:
   1. Run both "Getting started" and "Start Randovania" steps. 
   2. Activate the virtual env. Check start_client.bat/sh for details.
   3. Run `python -m pip install -r requirements.txt`.
   4. Run `python tools/prepare_dev_server_config.py` once.
   5. If you wish to use any Discord functionality, you'll need to create an app in Discord 
   and fill both ids in `tools/dev-server-configuration.json`.  
   6. Run the server and client. You can this on
      1. Windows with `tools/start_dev_server.bat` for the server and `tools/start_debug_client.bat` for the client
      2. Linux/macOS with `tools/start_dev_server.sh` for the server and `tools_start_debug_client.sh` for client
   
This repository uses [pre-commit](https://pre-commit.com/). The hook is automatically configured with 
the `prepare_virtual_env` scripts.

Suggested IDE: [PyCharm Community](https://www.jetbrains.com/pycharm/download/)

_*If using Visual Studio Code, make sure to set `"qtForPython.uic.liveExecution.enabled": false` when using the QT extension._

# Documentation

- Unfamiliar with a term? Check the [glossary](docs/Glossary.md).
- Adding a new game? Check the [dedicated guide](docs/New%20Game.md).
- Changing a data format? Check the [migrations documentation](docs/Migrations.md).
- Working with the logic database?
  - [Read how the database is organized](docs/Database%20Format.md).
  - Read the [unfinished script for a video](docs/Database%20Editor.md) on how to use the data editor.
